#include <Adafruit_NeoPixel.h>
#include <Wire.h> // must be included here so that Arduino library object file references work
#include <RtcDS3231.h>
#include <EepromAt24C32.h>
#include "Arduino.h"
#include "DFRobotDFPlayerMini.h"

RtcDS3231<TwoWire> Rtc(Wire);
EepromAt24c32<TwoWire> RtcEeprom(Wire); //for memory function to work

DFRobotDFPlayerMini myDFPlayer;  //DFPlayer object maken
//DFPlayer mini RX naar TX op arduino met 1kohm tussen; spk1 & spk2 naar speaker

#define PIN 6
#define LEDS 64

/* Settings for LED strips*/
// Parameter 1 = number of pixels in strip
// Parameter 2 = Arduino pin number (most are valid)
// Parameter 3 = pixel type flags, add together as needed
//   NEO_GRB     Pixels are wired for GRB bitstream (most NeoPixel products)
//   NEO_KHZ800  800 KHz bitstream (most NeoPixel products WS2812 LEDs)
Adafruit_NeoPixel strip = Adafruit_NeoPixel(LEDS, PIN, NEO_GRB + NEO_KHZ800);

/* Settings for Real time clock
  // CONNECTIONS:
  // DS3231 SDA --> SDA
  // DS3231 SCL --> SCL
  // DS3231 VCC --> 3.3v or 5v
  // DS3231 GND --> GND
  // SQW --->  (Pin19) Don't forget to pullup (4.7k to 10k to VCC)*/


#define RtcSquareWavePin 19 // Mega2560
#define RtcSquareWaveInterrupt 4 // Mega2560

// marked volatile so interrupt can safely modify them and
// other code can safely read and modify them
volatile uint16_t interuptCount = 0;
volatile bool interuptFlag = false;


// constants won't change. They're used here to set pin numbers:
const byte buttonUp = 12;     // the number of the pushbutton pin
const byte buttonDown = 11;     // the number of the pushbutton pin
//const int buttonMode = 8;     // the number of the pushbutton pin
const byte buttonAlarm = 7;     // the number of the pushbutton pin
const byte AlarmLed = 9;
const byte ModeLed = 10;
const byte ModeLed2 = 8;
const byte SpeakerPin = 3;

const bool UseLDR = true;
const byte BrightnessMin = 1;
const byte BrightnessMax = 200;
const byte LedBrightnessMin = 2;
const byte LedBrightnessMax = 200;
const byte buttonDelay = 200;
const String nummers[] = {"TWAALF", "EEN", "TWEE", "DRIE", "VIER", "VIJF", "ZES", "ZEVEN", "ACHT", "NEGEN", "TIEN", "ELF", "TWAALF"};
//, "DERTIEN","VIERTIEN","VIJFTIEN","ZESTIEN","ZEVENTIEN","ACHTTIEN","NEGENTIEN","TWINTIG"};
//const String maanden[] ={"","JA","FEB","MAA","APR","MEI","JUN","JUL","AUG","SEP","OKT","NOV","DEC"};
/*const String Voor = "VOOR";
  const String Na = "Na";
  const String Kwart = "KWART";
  const String Half = "HALF";
  const String Twintig = "TWINTIG";
*/

const byte VerjaardagMP3 = 14;
const uint16_t stringAddr = 64; // stored on page boundary

// constant variable containing the letters of the clockface
const String ClockFace = String("HETMISDTKWARVIENTBIJFGNAVOORHALFDNTWZEVRPHIAGERNALJFCHTSDUURAPMY");
byte Color1[] = {50, 120 , 255}; //blauwachtig
byte Color2[] = {30, 200, 90}; //groenachtig
byte Color3[] = {255, 100, 50}; //roodachtig
byte NightColor1[] = {0, 1 , }; //groen
byte NightColor2[] = {1, 0, 0}; //rood


// variables will change:
byte buttonState1 = 0;         // variable for reading the pushbutton status
byte buttonState2 = 0;
byte AlarmButtonState = 0;
byte ModeButtonState = 0;
int Uren = 1;
int Minuten = 1;
byte LoopCounter = 1;
byte BrightnessPreset = 85;
byte LedBrightnessPreset = 2;
int AlarmUren = 7;
int AlarmMinuten = 5;
int NuUren;
int NuMinuten;
int NuJaar;
byte NuMaand;
byte NuDag;
byte NuSeconden;
byte VerjaardagVandaag = 4;
byte VerjaardagVervers = 20;  // verwisseltijd in seconden, maximum is dertig
byte VolumeCounter = 0; // Counter om volume an alarm geleidelijk te laten toenemen
byte HuidigVolume = 3; // Huidig Volume tijdens beginnend alarm
int MaxVolume = 28; // Maw Volume
int MP3Choice = 1; // Maw Volume


const String VerjaardagNamen[] = {"", "Jan", "Kathleen", "Daan", "Ward"};
const int VerjaardagMaanden[] = {"", 4, 5, 1, 5};
const int VerjaardagDagen[] = {"", 26, 24, 23, 20};


bool AlarmState = false;
bool AlarmAan = false;
bool RainbowMode = false;
bool NachtMode = false;
bool DFPlayerPlaying = false;

String TekstTeTonen, VorigeTijd;
int AlarmData[] = {7, 0, 25, 2}; //for memory function
long firstPixelHue = 0; //Voor kleureneffect

void InteruptServiceRoutine()
{
  // since this interupted any other running code,
  // don't do anything that takes long and especially avoid
  // any communications calls within this routine
  interuptCount++;
  interuptFlag = true;
}


void setup() {

  strip.begin();
  strip.show();  //Zet hier alle pixels op uit.
  //strip.setBrightness(250); // Set BRIGHTNESS to about 1/5 (max = 255)

  // initialize the pushbutton pin as an input:
  pinMode(buttonUp, INPUT);
  pinMode(buttonDown, INPUT);
  pinMode(buttonAlarm, INPUT);
  //pinMode(buttonMode, INPUT);
  pinMode(AlarmLed, OUTPUT);
  pinMode(ModeLed, OUTPUT);

  // set the interupt pin of DS3231 to input mode & start RTC
  pinMode(RtcSquareWavePin, INPUT);

  Rtc.Begin();

  // Open serial communications and wait for port to open:
  Serial.begin(9600);
  while (!Serial) {
    ; // wait for serial port to connect. Needed for native USB port only
  }
  myDFPlayer.begin(Serial);  //DFPlayer aan serial koppelen
  myDFPlayer.volume(28);  //Set volume value. From 0 to 30
  //myDFPlayer.play(2);  //Play the first mp3

  // RTC initialisatie
  RtcDateTime compiled = RtcDateTime(__DATE__, __TIME__);

  if (!Rtc.IsDateTimeValid())
  {
    if (Rtc.LastError() != 0)
    {
      // we have a communications error
      // see https://www.arduino.cc/en/Reference/WireEndTransmission for
      // what the number means
      //Serial.print("RTC communications error = ");
      //Serial.println(Rtc.LastError());
    }
    else
    {
      //Serial.println("RTC lost confidence in the DateTime!");
      Rtc.SetDateTime(compiled);
    }
  }

  if (!Rtc.GetIsRunning())
  {
    //Serial.println("RTC was not actively running, starting now");
    Rtc.SetIsRunning(true);
  }

  RtcDateTime now = Rtc.GetDateTime();
  if (now < compiled)
  {
    //Serial.println("RTC is older than compile time!  (Updating DateTime)");
    Rtc.SetDateTime(compiled);
  }

  Rtc.Enable32kHzPin(false);
  Rtc.SetSquareWavePin(DS3231SquareWavePin_ModeAlarmBoth);

  // Tijd ook al direkt tonen op de LED matrix
  now = Rtc.GetDateTime();
  TekstTeTonen = UurNaarTekst (now.Hour(), now.Minute(), true);
  TekstNaarLED(TekstTeTonen, 150, 0);
  VorigeTijd = TekstTeTonen;

  /* Store something in memory on the Eeprom
    // store starting address of string
    RtcEeprom.SetMemory(0, stringAddr);
    AlarmData[0] = 19;
    AlarmData[1] = 05;
    // store the string, nothing longer than 32 bytes due to paging
    uint8_t written = RtcEeprom.SetMemory(stringAddr, (const uint8_t*)AlarmData, sizeof(AlarmData) - 1); // remove the null terminator strings add
    // store the length of the string
    RtcEeprom.SetMemory(1, written); */



}


void loop() {

  delay(buttonDelay);
  buttonState1 = digitalRead(buttonUp); // Read buttons
  buttonState2 = digitalRead(buttonDown);
  AlarmButtonState  = digitalRead(buttonAlarm);
  //ModeButtonState  = digitalRead(buttonMode);

  RtcDateTime now = Rtc.GetDateTime();
  NuUren = now.Hour();
  NuMinuten = now.Minute();
  NuSeconden = now.Second();
  NuMaand = now.Month();
  NuDag = now.Day();
  NuJaar = now.Year();

  //Opzoeken of vandaag iemand jarig is
  VerjaardagVandaag = 0;
  for (int PersoonX = 1; PersoonX < 5; PersoonX++) {
    if ((NuMaand == VerjaardagMaanden[PersoonX]) && (NuDag == VerjaardagDagen[PersoonX])) {
      VerjaardagVandaag = PersoonX;
    }
  }

  if (UseLDR) { //Adjust Brightness according to LDR value
    int sensorValue = analogRead(A0); // Read LDR  pin verbonden met ground over 4.7kohm weerstand (klok 2)
    if (sensorValue > 100) {
      sensorValue = 100;
    }
    LedBrightnessPreset = map (sensorValue, 0, 100, LedBrightnessMin, LedBrightnessMax);
    BrightnessPreset = map (sensorValue, 0, 100, BrightnessMin, BrightnessMax);

    if (BrightnessPreset == BrightnessMin) {
      NachtMode = true;
    }
    else {
      NachtMode = false;
    }
    /*Serial.print(sens<orValue);
      Serial.print("  ");
      Serial.print(LedBrightnessPreset);
      Serial.print("  ");
      Serial.println(BrightnessPreset);*/
  }


  // check if the pushbutton is pressed. If it is, the buttonState is HIGH:
  if (buttonState1 == HIGH && buttonState2 == HIGH) {
    TijdInstellen("e", NuUren, NuMinuten , 1);
    RtcDateTime now = Rtc.GetDateTime();
    NuUren = now.Hour();
    NuMinuten = now.Minute();
  }
  if (AlarmButtonState == HIGH ) { // Als alarmknop ingedrukt is dan eerst wachten om te zien of die niet ingedrukt blijft, indien zo alarm uitzetten of RainbowMode aan zetten
    //Serial.println ("alarmknop");
    delay(buttonDelay * 2);
    AlarmButtonState  = digitalRead(buttonAlarm);
    if (AlarmButtonState == HIGH and !AlarmState) {  //Als er alarmLED uit is en alarmknop ingeduwd blijft de rainbow modus aktiveren
      RainbowMode = true;
      //VorigeTijd = "rainbow";
      rainbow();
      while (AlarmButtonState == HIGH) {
        AlarmButtonState  = digitalRead(buttonAlarm); //wachten tot knop gelast wordt
      }
      //Serial.println ("RainbowMode aan");
    }
    else if (AlarmButtonState == HIGH and AlarmAan) { // Als knopnog steeds ingedrukt is en alarm staat nog steeds aan: alarm uitzetten
      AlarmAan = false; // RainbowMode uitzetten
      myDFPlayer.stop();
      DFPlayerPlaying = false;
      strip.setBrightness(255);
      analogWrite(AlarmLed, 0);
      AlarmState = false; //Alarm uitzetten
    }
    else if (RainbowMode) { // Als rainbowMode aktief is deze uitzetten
      RainbowMode = false; // RainbowMode uitzetten
      VorigeTijd = "rainbow";
      //Serial.println ("RainbowMode uit");
    }
    else if (AlarmAan) {  //Als alarm aan het afgaan is dit uizetten & instellen voor binnen 5 minuten
      AlarmAan = false; //Alarm uitzetten
      myDFPlayer.stop();
      DFPlayerPlaying = false;
      AlarmMinuten = AlarmMinuten + 5;
      strip.setBrightness(255);
      if (AlarmMinuten >= 60) {
        AlarmUren++;
        AlarmMinuten = AlarmMinuten - 60;
        /*Serial.print (NuUren);
          Serial.print (":");
          Serial.print (NuMinuten);
          Serial.print (" - ");
          Serial.print (TekstTeTonen);
          Serial.print (" alarm: ");
          Serial.print (AlarmUren);
          Serial.print (":");
          Serial.println (AlarmMinuten);
        */

      }
    }
    else if (!AlarmState) { // Als alarm uit is dit aanzetten
      AlarmState = true; //Toggle alarm state
      //Serial.println ("Alarm Aan");
      AlarmUitlezen();
    }
    else {
      AlarmState = false;
      //Serial.println ("Alarm Uit");
    }

  }

  //Set alarm leds according to alarm state
  if (AlarmState) {
    analogWrite(AlarmLed, LedBrightnessPreset);
  }
  else {
    analogWrite(AlarmLed, 0);
    AlarmAan = false; //Alarm uitzetten
    //myDFPlayer.stop();
    //DFPlayerPlaying = false;
  }

  // alarm laten afgaan op juiste moment
  if ((AlarmState) and (NuUren == AlarmUren) and (NuMinuten == AlarmMinuten) and !DFPlayerPlaying) {
    myDFPlayer.volume(1);  //Alarm op minimum zetten
    VolumeCounter = 0;
    HuidigVolume = 1;
    if (VerjaardagVandaag != 0) {   // bij verjaardagen happy birthday spelen anders gekozen nummer
      myDFPlayer.loop(VerjaardagMP3);  //play the mp3
      Serial.println (VerjaardagMP3);

    }
    else {
      myDFPlayer.loop(MP3Choice);  //play the mp3
    }
    DFPlayerPlaying = true;
    AlarmAan = true;
    strip.setBrightness(10); // Set BRIGHTNESS to about 1/5 (max = 255)
  }

  // Als het alarm aan is volume geleidelijk toe laten nemen
  if (AlarmAan) {
    VolumeCounter++;
    if (VolumeCounter >= 15) {
      VolumeCounter = 0;
      if (HuidigVolume <  MaxVolume) {
        HuidigVolume++;
        //Serial.println (HuidigVolume);
        myDFPlayer.volume(HuidigVolume);
        strip.setBrightness(HuidigVolume * 7 ); // brightness van scherm ook laten toenemen

      }
    }
    //tone(SpeakerPin , 330, buttonDelay) ; voor gebruik met speaker zonder DFPlayer


  }

  // Goede tekst samenstellen om op het scherm te tonen
  if ((VerjaardagVandaag > 0) && (NuSeconden % (VerjaardagVervers * 2) > VerjaardagVervers) ) {
    switch ((int)round((NuSeconden % 6 - 0.2 ) / 2)) { //moeilijke formule om om de 2 seconden te verwisselen
      case 0:
        TekstTeTonen = VerjaardagNamen[VerjaardagVandaag];
        break;
      case 1:
        TekstTeTonen = "Happy";
        break;
      case 2:
        TekstTeTonen = " Birthday";
        break;
    }
    //TekstTeTonen = "Happy Birthday";
    //TekstTeTonen = VerjaardagNamen[VerjaardagVandaag];
  }
  else if (AlarmAan)  {
    rainbow();
  }
  else if (RainbowMode)  {
    rainbow();
  }
  else {
    TekstTeTonen = UurNaarTekst (NuUren, NuMinuten, true);
  }

  if (TekstTeTonen  != VorigeTijd ) { //enkel scherm verversen wanneer tijd veranderd is
    TekstNaarLED(TekstTeTonen, 150, 0);
    VorigeTijd = TekstTeTonen;
    //Serial.println (now.Year());
    /*Serial.print (NuUren);
      Serial.print (":");
      Serial.print (NuMinuten);
      Serial.print (" - ");
      Serial.print (TekstTeTonen);
      Serial.print (" alarm: ");
      Serial.print (AlarmUren);
      Serial.print (":");
      Serial.println (AlarmMinuten);
    */
    /*Serial.print (NuDag);
      Serial.print("-");
      Serial.print (NuMaand);
      Serial.print("-");
      Serial.println (NuJaar);
    */
  }
}


void setPixelColor2( uint16_t n, uint8_t r, uint8_t g, uint8_t b, uint16_t brightness) {
  strip.setPixelColor(n, (brightness * r / 255) , (brightness * g / 255), (brightness * b / 255));
}


void AlarmUitlezen() {
  // read data from alarm
  // get the offset we stored our data from address zero
  uint8_t address = RtcEeprom.GetMemory(0);
  if (address != stringAddr)
  {
    //Serial.print("address didn't match ");
    //Serial.println(address);
    AlarmUren = 7;
    AlarmMinuten = 5;
    MaxVolume = 8;
    MP3Choice = 3;
  }
  else
  {
    // get the size of the data from address 1
    uint8_t count = RtcEeprom.GetMemory(1);
    uint8_t buff[64];

    // get our data from the address with the given size
    uint8_t gotten = RtcEeprom.GetMemory(address, buff, count);
    /*Serial.print ("buff0: ");
      Serial.print(buff[0]);//uren
      Serial.print(" 2:");
      Serial.print(buff[2]); //minuten
      Serial.print(" 4:");
      Serial.print(buff[4]); //volume
      Serial.print(" 6:");
      Serial.print(buff[6]); //MP3keuze
    */



    AlarmUren = buff[0];
    AlarmMinuten = buff[2];
    MaxVolume = buff[4];
    MP3Choice = buff[6];

  }
  TekstTeTonen = UurNaarTekst (AlarmUren, AlarmMinuten, false);
  TekstNaarLED(TekstTeTonen, 0, 0);
  TijdInstellen("e", AlarmUren, AlarmMinuten , 2); // alarm tijd instellen


}


String TijdInstellen(String InputTekst, int UurStart , int MinutenStart , int TI_Mode) {   // TI_Mode 1 = tijd instellen, 2 = alarmtijd instellen
  int Teller = MinutenStart;
  int TellerOud = 30;
  int Timer =  0;
  int TimerMillis = 0;
  bool LedState = false;
  bool SkipNaarVolgende = false;
  int BlinkLED1 = ModeLed;
  int BlinkLED2 = ModeLed2;
  Timer =  millis();
  TimerMillis =  millis();
  Uren = UurStart;
  Serial.println("Tijd instellen functie");
  if (TI_Mode == 2) {
    BlinkLED1 = AlarmLed;
  }
  Serial.println("uren instellen");
  // Uren instellen
  analogWrite(AlarmLed, 0); // alarmLED uitzetten tijdens instellen uur
  if (TI_Mode==1){
  TekstNaarLED("UUR", 0, 57);
  delay(2000);
  TekstTeTonen = UurNaarTekst (UurStart, MinutenStart, false);
  TekstNaarLED(TekstTeTonen, 0, 3);
  }


  while (TimerMillis - Timer < 3000 and !SkipNaarVolgende) {
    TimerMillis =  millis();
    LedState = !LedState;
    if (LedState) {
      digitalWrite(BlinkLED1, HIGH);   // turn the LED on (HIGH is the voltage level)
    }
    else {
      digitalWrite(BlinkLED1, LOW);   // turn the LED off
    }
    // read the state of the pushbutton value:
    delay(buttonDelay);
    buttonState1 = digitalRead(buttonUp);
    buttonState2 = digitalRead(buttonDown);
    // check if the pushbutton is pressed. If it is, the buttonState is HIGH:
    if (buttonState1 == HIGH and buttonState2 == LOW ) {
      Uren++;
      Timer = millis();
    }
    if (buttonState2 == HIGH and buttonState1 == LOW) {
      Uren--;
      Timer = millis();
    }
    if (Uren < 0) {
      Uren = 23;
    }
    if (Uren > 23) {
      Uren = 0;
    }
    if (buttonState2 == HIGH and buttonState1 == HIGH) {
      //TekstNaarLED("TOON", 0, 3);
      SkipNaarVolgende = true;

    }
    if (Uren != TellerOud) {
      TellerOud = Uren;
      TekstTeTonen = UurNaarTekst (Uren, MinutenStart, false);
      TekstNaarLED(TekstTeTonen, 0, 3);
    }
  }

  digitalWrite(ModeLed, LOW);   // turn the LED off
  analogWrite(AlarmLed, 0); // alarmLED uitzetten tijdens instellen uur

  // Minuten instellen
  TimerMillis =  millis();
  Timer =  millis();

  //Serial.println("minuten instellen");
  while (TimerMillis - Timer < 3000 and !SkipNaarVolgende) {
    TimerMillis =  millis();
    LedState = !LedState;
    if (LedState) {
      digitalWrite(BlinkLED2, HIGH);   // turn the LED off
    }
    else {
      digitalWrite(BlinkLED2, LOW);   // turn the LED off
    }
    // read the state of the pushbutton value:
    delay(buttonDelay);
    buttonState1 = digitalRead(buttonUp);
    buttonState2 = digitalRead(buttonDown);
    // check if the pushbutton is pressed. If it is, the buttonState is HIGH:
    if (buttonState1 == HIGH and buttonState2 == LOW) {
      Teller = Teller + 5;
      Timer = millis();
      if (Teller >= 60) {
        Teller = 0;
        Uren++;
      }
    }
    if (buttonState2 == HIGH and buttonState1 == LOW) {
      Teller = Teller - 5;
      Timer = millis();
      if (Teller < 0) {
        Teller = 55;
        Uren--;
      }
    }
    if (Uren < 0) {
      Uren = 23;
    }
    if (Uren > 23) {
      Uren = 0;
    }
    if (buttonState2 == HIGH and buttonState1 == HIGH) {
      SkipNaarVolgende = true;
    }

    if (Teller != TellerOud) {
      TellerOud = Teller;
      Minuten = Teller;
      TekstTeTonen = UurNaarTekst (Uren, Minuten, false);
      VorigeTijd = "x";
      TekstNaarLED(TekstTeTonen, 0, 3);
    }
  }

  if (SkipNaarVolgende == true and TI_Mode == 2 ) { // ALS Dubbelklik naar volgende instelling skippen
    TekstNaarLED("TOON", 0, 3);
    delay(1000);
    MP3Kiezen();
  }
  if (SkipNaarVolgende == true and TI_Mode == 1) { // ALS Dubbelklik naar volgende instelling skippen
    TekstNaarLED("DATUM", 0, 3);
    delay(1000);
    DatumInstellen();
  }


  if (TI_Mode == 1) { // Tijd wegschrijven naar DS3231
    RtcDateTime now = Rtc.GetDateTime();
    /*
      Serial.println (NuJaar);
      Serial.println (NuMaand);
      Serial.println (NuDag);
      Serial.println (Uren);
      Serial.println (Minuten);
    */
    Rtc.SetDateTime(RtcDateTime(NuJaar, NuMaand, NuDag, Uren, Minuten, 00));
  }
  else if (TI_Mode == 2) { // Alarmtijd wegschrijven naar vast geheugen DS3231
    // Store something in memory on the Eeprom
    // store starting address of string
    //Serial.print(MaxVolume);
    //Serial.println(MP3Choice);
    RtcEeprom.SetMemory(0, stringAddr);
    AlarmData[0] = Uren;
    AlarmData[1] = Minuten;
    AlarmData[2] = MaxVolume;
    AlarmData[3] = MP3Choice;
    // store the string, nothing longer than 32 bytes due to paging
    uint8_t written = RtcEeprom.SetMemory(stringAddr, (const uint8_t*)AlarmData, sizeof(AlarmData) - 1); // remove the null terminator strings add
    // store the length of the string
    RtcEeprom.SetMemory(1, written);
    AlarmUren = Uren;
    AlarmMinuten = Minuten;

  }
  //analogWrite(ModeLed, 0);
  //analogWrite(AlarmLed, 0);
  digitalWrite(BlinkLED1, LOW);   // turn the LED off
  digitalWrite(BlinkLED2, LOW);   // turn the LED off



  return "Tijd OK";
}


String TekstNaarLED(String InputTekst, int animation, int StartLed) {
  //String outputi  = String("Hello World..!");
  byte ClockFaceIndex = StartLed;
  char Tekeni;
  byte IndexTekeni = 0;
  byte Kleur = 1;


  InputTekst.toUpperCase();
  strip.clear();         //   Set all pixels in RAM to 0 (off)
  if (InputTekst.length() >= 1) {
    // loop from the lowest pin to the highest:
    for (int HuidigeIndex = 0; HuidigeIndex <= InputTekst.length() - 1; HuidigeIndex++) {
      // teken op deze positie
      Tekeni = InputTekst.charAt(HuidigeIndex);
      IndexTekeni = ClockFace.indexOf(Tekeni, ClockFaceIndex);
      if (isWhitespace(Tekeni)) {
        // Als character is spatie kleur enkel kleur veranderen
        //Serial.println("Kleur veranderen");
        Kleur = Kleur + 1;
      }

      else if (IndexTekeni == -1) {
        // skip unknown characters
        //Serial.print("x");
        //Serial.print(Tekeni);
        //Serial.println(IndexTekeni);

      }
      else {
        // indexOf() returns the position (i.e. index) of a particular character in a string
        if ( Kleur % 2 == 1) {
          if (NachtMode == true) {
            strip.setPixelColor(IndexTekeni, NightColor1[0], NightColor1[1], NightColor1[2]);
          }
          else {
            setPixelColor2(IndexTekeni, Color1[0], Color1[1], Color1[2], BrightnessPreset);
          }
        }
        else {
          if (NachtMode == true) {
            strip.setPixelColor(IndexTekeni, NightColor2[0], NightColor2[1], NightColor2[2]);
          }
          else {
            setPixelColor2(IndexTekeni, Color2[0], Color2[1], Color2[2], BrightnessPreset);
          }

        }
        delay(animation);
        if (animation > 0) {
          strip.show();
        }

        //Serial.print(Tekeni);
        //Serial.print(" LED aan ");
        //Serial.println(IndexTekeni);
        ClockFaceIndex = IndexTekeni + 1;
      }
    }
  }
  strip.show(); //Laat de kleuren zien!
  return "x";
}


String UurNaarTekst(int Uur, int Min, bool HetIs) {
  String outputi  = String("x");
  bool PM = false;
  String Voor = "VOOR";
  String Na = "Na";
  String Kwart = "KWART";
  String Half = "HALF";
  String Twintig = "TWINTIG";
  if (Uur >= 12) {
    PM = true;
  }
  if (Min >= 25) {
    Uur = Uur + 1;
  }
  if (Uur > 12)  {
    Uur = Uur - 12;
  }
  //String UurTekst = nummers[Uur];
  Min = floor(Min / 5);
  switch (Min) {
    case 0: outputi = nummers[Uur] + " uur" ;    break;
    case 1: outputi = nummers[5] + " " + Na + " " + nummers[Uur];    break;
    case 2: outputi = nummers[10] + " " + Na + " " + nummers[Uur];    break;
    case 3: outputi = Kwart      + " " + Na + " " + nummers[Uur];    break;
    case 4: outputi = Twintig    + " " + Na + " " + nummers[Uur];    break;
    case 5: outputi = nummers[5] + " " + Voor + " " + Half + " " + nummers[Uur];    break;
    case 6: outputi = Half + " " + nummers[Uur];    break;
    case 7: outputi = nummers[5] + " " + Na   + " " + Half + " " + nummers[Uur];    break;
    case 8: outputi = Twintig      + " " + Voor + " " + nummers[Uur];    break;
    case 9: outputi = Kwart        + " " + Voor + " " + nummers[Uur];    break;
    case 10: outputi = nummers[10] + " " + Voor + " " + nummers[Uur];    break;
    case 11: outputi = nummers[5]  + " " + Voor + " " + nummers[Uur];    break;
    default: outputi = "ERR";    break;
  }
  if (PM) {
    outputi = outputi + " PM";
  }
  else {
    outputi = outputi + " AM";
  }
  if (HetIs) {
    outputi = "HET IS " + outputi;
  }
  //Uitzonderingen voor middag en middernacht
  if (Uur == 12 && Min == 0) {
    outputi = "MIDDAG";
  }
  if (Uur == 0  && Min == 0) {
    outputi = "MIDDERNACHT";
  }
  return outputi;
}


// Rainbow cycle along whole strip.
void rainbow() {
  // Hue of first pixel runs 3 complete loops through the color wheel.
  // Color wheel has a range of 65536 but it's OK if we roll over, so
  // just count from 0 to 3*65536. Adding 256 to firstPixelHue each time
  // means we'll make 3*65536/256 = 768 passes through this outer loop:
  firstPixelHue = firstPixelHue + 1000;
  //Serial.print("firstPixelHue = ");
  //Serial.println(firstPixelHue );

  for (int i = 0; i < strip.numPixels(); i++) { // For each pixel in strip...
    // Offset pixel hue by an amount to make one full revolution of the
    // color wheel (range of 65536) along the length of the strip
    // (strip.numPixels() steps):
    int pixelHue = firstPixelHue + (i * 65536L / strip.numPixels());
    // strip.ColorHSV() can take 1 or 3 arguments: a hue (0 to 65535) or
    // optionally add saturation and value (brightness) (each 0 to 255).
    // Here we're using just the single-argument hue variant. The result
    // is passed through strip.gamma32() to provide 'truer' colors
    // before assigning to each pixel:
    strip.setPixelColor(i, strip.gamma32(strip.ColorHSV(pixelHue)));
  }
  strip.show(); // Update strip with new contents
}

void MP3Kiezen() {   // TI_Mod
  byte Teller = 1;
  byte TellerOud = 30;
  int Timer =  0;
  int TimerMillis = 0;
  bool LedState = false;
  bool SkipNaarVolgende = false;
  byte  BlinkLED1 = ModeLed;
  byte  BlinkLED2 = ModeLed2;
  Timer =  millis();
  TimerMillis =  millis();

  //Serial.println("MP3 instellen");
  analogWrite(AlarmLed, 0); // alarmLED uitzetten tijdens instellen
  myDFPlayer.volume(MaxVolume);  //Alarmvolume instellen
  Teller = MP3Choice;
  TellerOud = 19;

  while (TimerMillis - Timer < 5000 and SkipNaarVolgende == false) {
    TimerMillis =  millis();
    LedState = !LedState;
    if (LedState) {
      digitalWrite(BlinkLED1, HIGH);   // turn the LED on (HIGH is the voltage level)
    }
    else {
      digitalWrite(BlinkLED1, LOW);   // turn the LED off
    }
    // read the state of the pushbutton value:
    delay(buttonDelay);
    buttonState1 = digitalRead(buttonUp);
    buttonState2 = digitalRead(buttonDown);
    // check if the pushbutton is pressed. If it is, the buttonState is HIGH:
    if (buttonState1 == HIGH and buttonState2 == LOW) {
      Teller++;
      Timer = millis();
    }
    if (buttonState2 == HIGH and buttonState1 == LOW) {
      Teller--;
      Timer = millis();
    }
    if (Teller <= 0) {
      Teller = 12;
    }
    if (Teller > 16) {
      Teller = 1;
    }
    if (buttonState2 == HIGH and buttonState1 == HIGH) {
      //TekstNaarLED("TOON", 0, 3);
      SkipNaarVolgende = true;

    }
    if (Teller != TellerOud) {
      TellerOud = Teller;
      TekstTeTonen = nummers[Teller];
      TekstNaarLED(TekstTeTonen, 0, 31);
      myDFPlayer.play(Teller);
      Serial.println(Teller);
    }
  }

  myDFPlayer.stop();
  MP3Choice = Teller;
  myDFPlayer.loop(MP3Choice);  //Loop the mp3
  Timer =  millis();
  TimerMillis =  millis();
  Teller = MaxVolume / 3;
  TellerOud = 19;
  TekstNaarLED("VOLUME", 0, 1);
  delay(2000);
  TekstNaarLED("Teller", 0, 31);


  //Serial.println("Volume instellen");
  analogWrite(AlarmLed, 0); // alarmLED uitzetten tijdens instellen

  while (TimerMillis - Timer < 5000 and SkipNaarVolgende == false) {
    TimerMillis =  millis();
    LedState = !LedState;
    if (LedState) {
      digitalWrite(BlinkLED1, HIGH);   // turn the LED on (HIGH is the voltage level)
    }
    else {
      digitalWrite(BlinkLED1, LOW);   // turn the LED off
    }
    // read the state of the pushbutton value:
    delay(buttonDelay);
    buttonState1 = digitalRead(buttonUp);
    buttonState2 = digitalRead(buttonDown);
    // check if the pushbutton is pressed. If it is, the buttonState is HIGH:
    if (buttonState1 == HIGH and buttonState2 == LOW) {
      Teller++;
      Timer = millis();
    }
    if (buttonState2 == HIGH and buttonState1 == LOW) {
      Teller--;
      Timer = millis();
    }
    if (Teller <= 0) {
      Teller = 10;
    }
    if (Teller > 10) {
      Teller = 1;
    }

    if (Teller != TellerOud) {
      TellerOud = Teller;
      TekstTeTonen = nummers[Teller];
      TekstNaarLED(TekstTeTonen, 0, 31);
      myDFPlayer.volume(Teller * 3); //Alarmvolume instellen

    }
  }

  myDFPlayer.stop();
  MaxVolume = Teller * 3;


  digitalWrite(ModeLed, LOW);   // turn the LED off
  analogWrite(AlarmLed, 0); // alarmLED uitzetten tijdens instellen uur
  digitalWrite(BlinkLED1, LOW);   // turn the LED off
  digitalWrite(BlinkLED2, LOW);   // turn the LED off

  if (SkipNaarVolgende == true) { // ALS Dubbelklik naar volgende instelling skippen
    TekstNaarLED("TOON", 0, 3);
    delay(5000);
  }
  VorigeTijd = "y";

}

void DatumInstellen() {   // TI_Mod
  byte Teller = 1;
  byte TellerOud = 30;
  int Timer =  0;
  int TimerMillis = 0;
  bool LedState = false;
  bool SkipNaarVolgende = false;
  byte  BlinkLED1 = ModeLed;
  byte  BlinkLED2 = ModeLed2;
  Timer =  millis();
  TimerMillis =  millis();
  TekstNaarLED("JAAR", 0, 1);
  delay(2000);
  //Serial.println("Jaar instellen");


  analogWrite(AlarmLed, 0); // alarmLED uitzetten tijdens instellen

  if (NuJaar < 2020) {
    NuJaar = 2019;
  }
  Teller = NuJaar - 2000;
  TellerOud = 18;

  while (TimerMillis - Timer < 5000 and SkipNaarVolgende == false) {
    TimerMillis =  millis();
    LedState = !LedState;
    if (LedState) {
      digitalWrite(BlinkLED1, HIGH);   // turn the LED on (HIGH is the voltage level)
    }
    else {
      digitalWrite(BlinkLED1, LOW);   // turn the LED off
    }
    // read the state of the pushbutton value:
    delay(buttonDelay);
    buttonState1 = digitalRead(buttonUp);
    buttonState2 = digitalRead(buttonDown);
    // check if the pushbutton is pressed. If it is, the buttonState is HIGH:
    if (buttonState1 == HIGH and buttonState2 == LOW) {
      Teller++;
      Timer = millis();
    }
    if (buttonState2 == HIGH and buttonState1 == LOW) {
      Teller--;
      Timer = millis();
    }
    if (Teller <= 18) {
      Teller = 19;
    }
    /*
      if (Teller > 12) {
      Teller = 1;
      }*/
    /*
      if (buttonState2 == HIGH and buttonState1 == HIGH) {
      //TekstNaarLED("TOON", 0, 3);
      SkipNaarVolgende = true;

      }
    */
    if (Teller != TellerOud) {
      TellerOud = Teller;
      TekstTeTonen = nummers[Teller];
      if (Teller == 19) {
        TekstTeTonen = "TIEN NEGEN";
      }
      if (Teller == 20) {
        TekstTeTonen = "TWINTIG";
      }
      if (Teller >= 21) {
        TekstTeTonen = "TWINTIG " + nummers[Teller - 20];
      }
      TekstNaarLED(TekstTeTonen, 0, 0);
      /*Serial.println(Teller);
        Serial.println(TekstTeTonen);
      */
    }
  }

  NuJaar = Teller + 2000;

  Timer =  millis();
  TimerMillis =  millis();
  TellerOud = 14;
  Teller = NuMaand;
  TekstNaarLED("MAAND", 0, 1);
  delay(2000);


  analogWrite(AlarmLed, 0); // alarmLED uitzetten tijdens instellen

  while (TimerMillis - Timer < 5000 and SkipNaarVolgende == false) {
    TimerMillis =  millis();
    LedState = !LedState;
    if (LedState) {
      digitalWrite(BlinkLED1, HIGH);   // turn the LED on (HIGH is the voltage level)
    }
    else {
      digitalWrite(BlinkLED1, LOW);   // turn the LED off
    }
    // read the state of the pushbutton value:
    delay(buttonDelay);
    buttonState1 = digitalRead(buttonUp);
    buttonState2 = digitalRead(buttonDown);
    // check if the pushbutton is pressed. If it is, the buttonState is HIGH:
    if (buttonState1 == HIGH and buttonState2 == LOW) {
      Teller++;
      Timer = millis();
    }
    if (buttonState2 == HIGH and buttonState1 == LOW) {
      Teller--;
      Timer = millis();
    }

    if (Teller <= 0) {
      Teller = 1;
    }
    if (Teller > 12) {
      Teller = 12;
    }

    if (Teller != TellerOud) {
      TellerOud = Teller;
      // TekstTeTonen = nummers[Teller];

      if (Teller >= 21) {
        TekstTeTonen = "TWINTIG " + nummers[Teller - 20];
      }
      else if (Teller == 20) {
        TekstTeTonen = "TWINTIG";
      }
      else if (Teller >= 13) {
        TekstTeTonen = "TIEN " + nummers[Teller - 10];
      }
      else
      {
        TekstTeTonen = nummers[Teller];
      }


      TekstNaarLED(TekstTeTonen, 0, 0);
      /*
        Serial.println(Teller);
        Serial.println(TekstTeTonen);
      */
    }
  }
  NuMaand = Teller;

  Timer =  millis();
  TimerMillis =  millis();
  TellerOud = 40;
  Teller = NuDag;
  TekstNaarLED("Dag", 0, 1);
  delay(2000);


  analogWrite(AlarmLed, 0); // alarmLED uitzetten tijdens instellen

  while (TimerMillis - Timer < 5000 and SkipNaarVolgende == false) {
    TimerMillis =  millis();
    LedState = !LedState;
    if (LedState) {
      digitalWrite(BlinkLED1, HIGH);   // turn the LED on (HIGH is the voltage level)
    }
    else {
      digitalWrite(BlinkLED1, LOW);   // turn the LED off
    }
    // read the state of the pushbutton value:
    delay(buttonDelay);
    buttonState1 = digitalRead(buttonUp);
    buttonState2 = digitalRead(buttonDown);
    // check if the pushbutton is pressed. If it is, the buttonState is HIGH:
    if (buttonState1 == HIGH and buttonState2 == LOW) {
      Teller++;
      Timer = millis();
    }
    if (buttonState2 == HIGH and buttonState1 == LOW) {
      Teller--;
      Timer = millis();
    }

    if (Teller <= 0) {
      Teller = 1;
    }
    if (Teller > 31) {
      Teller = 31;
    }

    if (Teller != TellerOud) {
      TellerOud = Teller;
      // TekstTeTonen = nummers[Teller];

      if (Teller >= 21) {
        TekstTeTonen = "TWINTIG " + nummers[Teller - 20];
      }
      else if (Teller == 20) {
        TekstTeTonen = "TWINTIG";
      }
      else if (Teller >= 13) {
        TekstTeTonen = "TIEN " + nummers[Teller - 10];
      }
      else
      {
        TekstTeTonen = nummers[Teller];
      }


      TekstNaarLED(TekstTeTonen, 0, 0);
      /*
        Serial.println(Teller);
        Serial.println(TekstTeTonen);
      */
    }
  }
  NuDag = Teller;


  digitalWrite(ModeLed, LOW);   // turn the LED off
  analogWrite(AlarmLed, 0); // alarmLED uitzetten tijdens instellen uur
  digitalWrite(BlinkLED1, LOW);   // turn the LED off
  digitalWrite(BlinkLED2, LOW);   // turn the LED off

  if (SkipNaarVolgende == true) { // ALS Dubbelklik naar volgende instelling skippen
    TekstNaarLED("TOON", 0, 3);
    delay(5000);
  }
  VorigeTijd = "y";

}

#define countof(a) (sizeof(a) / sizeof(a[0]))
